package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.core.api.iservice.busi.ICoreTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.controller.DemoAuthController;
import cn.estsh.i3plus.core.apiservice.serviceimpl.busi.SysLocaleLanguageService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authc.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


/**
 * @Description : 用户权限服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 21:01
 * @Modify :
 **/
@RestController
@Api(description="用户所有权限操作服务")
@RequestMapping(CoreBaseController.BASE_URL + "/auth")
public class AuthController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(DemoAuthController.class);

    @Autowired
    private ISystemLoginService systemLoginService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ICoreTreeService coreTreeService;

    @Autowired
    private SysLocaleLanguageService localeLanguageService;

    @GetMapping(value="/login")
    @ApiOperation(value="登陆",notes="登陆")
    public ResultBean login(HttpServletRequest request, String loginName, String loginPwd, String languageCode){
        LOGGER.info("用户登陆 loginName:{} loginPwd:{} languageCode:{}",loginName,loginPwd,languageCode);
        ResultBean result = null;
        try {
            ValidatorBean.checkNotNull(loginName,"用户名不能为空");
            ValidatorBean.checkNotNull(loginPwd,"密码不能为空");
            ValidatorBean.checkNotNull(languageCode,"语言不能为空");

            SessionUser user = userService.queryUserLogin(loginName,loginPwd,languageCode);

            String redisKey = CommonConstWords.USER_LOGIN_ERROR + "_" + user.getUser().getId();
            redisCore.deleteKey(redisKey);

            LOGGER.info("会员登陆：{}",user);
            AuthUtil.setSessionObject("languageCode",languageCode);
            result = new ResultBean(true, "", AuthUtil.getSessionUser());
            result.setUrl("/");
        } catch (IncorrectCredentialsException e) {
            // 密码错误
            Integer num = systemLoginService.doLoginPasswordError(loginName);
            num = CommonConstWords.USER_LOGIN_ERROR_MAX_NUM - num;
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_PASSWORD);
            if(num == 0){
                result.setErrorMsg("密码输入错误。用户已被锁定请联系管理员！");
            }else {
                result.setErrorMsg("密码输入错误。再输入错误"+ num +"次，用户将被锁定。");
            }
        } catch (CredentialsException e) {
            // 用户名或密码错误
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_NAME);
        } catch (LockedAccountException e) {
            // 账号已锁定
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_LOCKING);
        } catch (UnknownAccountException e) {
            // 用户信息不存在
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_INFO_NULL);
        } catch (AuthenticationException e) {
            // 系统异常
            result = ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
            result.setErrorMsg(e.getMessage());
        } catch (ImppBusiException e) {
            result = ResultBean.fail(e);
        } catch (Exception e) {
            result = ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode())
                        .setErrorMsg(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription());
        }
        return result;
    }

    /**
     *
     * @return
     */
    @GetMapping(value="/logout")
    @ApiOperation(value="登出",notes="登出")
    public ResultBean logout(){
        LOGGER.info("用户退出...");
        try {
            AuthUtil.logout();
            return new ResultBean(true,"").build();
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + ",so:" + e.getErrorSolution());
        }
    }

    /**
     *
     * @return
     */
    @GetMapping(value="/get-user-info")
    @ApiOperation(value="获取登录用户信息",notes="获取当前登录用户详细信息")
    public ResultBean getUserInfo(){
        LOGGER.info("获取用户信息...lang:{}",AuthUtil.getSessionAttribute("languageCode"));
        try {
            SessionUser sUser = AuthUtil.getSessionUser();
            LOGGER.info("用户：{}",sUser);
            return new ResultBean(true,"",sUser.getUserInfo());
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + ",so:" + e.getErrorSolution());
        }
    }

    /**
     *
     * @return
     */
    @GetMapping(value="/get-user")
    @ApiOperation(value="获取登录用户信息",notes="获取当前登录用户账号详细信息")
    public ResultBean getUser(){
        LOGGER.info("获取用户信息...lang:{}",AuthUtil.getSessionAttribute("languageCode"));
        try {
            SessionUser sUser = AuthUtil.getSessionUser();
            LOGGER.info("用户：{}",sUser);
            return new ResultBean(true,"",sUser.getUser());
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + ",so:" + e.getErrorSolution());
        }
    }

    @GetMapping(value="/find-menu-tree")
    @ApiOperation(value="获取登录用户功能权限信息",notes="获取当前登录用户的权限信息")
    public ResultBean getUserMenuTree(){
        try {
            List<SysMenu> listTree = coreTreeService.findSysMenuTreeByParentId(-1);

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(listTree);
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update-language")
    @ApiOperation(value="修改语言",notes="修改用户会话语言")
    public ResultBean updateLanguage(String code){
        try {
            SysLocaleLanguage language = localeLanguageService.getSysLocaleLanguageByCode(code);
            if(language != null){
                AuthUtil.setSessionLanguage(language.getLanguageCode());
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }else {
                return ResultBean.fail("操作失败").setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode()).setErrorMsg("语言信息不存在");
            }
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

}
