package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysRefExpressionTrigger;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTime;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTimeExpression;
import cn.estsh.i3plus.pojo.platform.repository.SysRefExpressionTriggerRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysTaskTimeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashSet;
import java.util.List;

/**
 * @Description : 定时任务业务接口实现
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.974
 * @Modify      :
 **/
@Service
public class SysTaskTimeService implements ISysTaskTimeService{

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskTimeService.class);

    @Autowired
    private SysTaskTimeRepository taskTimeRDao;

    @Autowired
    private SysRefExpressionTriggerRepository refExpressionTriggerRDao;

    @Autowired
    private Scheduler scheduler;

    @Override
    @ApiOperation(value = "修改定时任务")
    public void updateSysTaskTime(SysTaskTime taskTime, SysTaskTimeExpression taskTimeExpression) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        SysTaskTime oldSysTaskTime = taskTimeRDao.getById(taskTime.getId());

        // 更新关系表
        refExpressionTriggerRDao.updateByProperties(new String[]{"triggerName","triggerGroupName"},
                new String[]{oldSysTaskTime.getName(),taskTime.getGroupName()},
                "expressionId",oldSysTaskTime.getTimeExpressionId()
        );

        taskTimeRDao.update(taskTime);

        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(taskTime.getName(),taskTime.getGroupName());
            System.out.println(scheduler.getTrigger(triggerKey));

            if (!scheduler.checkExists(triggerKey)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("定时任务不存在")
                        .setErrorSolution("请重新选择")
                        .build();
            }

            // 添加定时任务与触发器到quartz表中
            CronScheduleBuilder schedBuilder = CronScheduleBuilder
                    .cronSchedule(taskTimeExpression.getExpressionContent())
                    .withMisfireHandlingInstructionDoNothing();
            TriggerBuilder<CronTrigger> trigger = TriggerBuilder.newTrigger()
                    .withIdentity(triggerKey)
                    .withDescription(taskTimeExpression.getExpressionDescription())
                    .withSchedule(schedBuilder)
                    .startAt(sdf.parse(taskTimeExpression.getExpressionStartDatetime()));
            if(taskTimeExpression.getExpressionEndDatetime() != null){
                trigger = trigger.endAt(sdf.parse(taskTimeExpression.getExpressionEndDatetime()));
            }

            JobKey jobKey = new JobKey(taskTime.getName(), taskTime.getGroupName());
            JobDetail jobDetail = scheduler.getJobDetail(jobKey);
            jobDetail.getJobBuilder().withDescription(taskTime.getTaskDescription());

            HashSet<Trigger> triggerSet = new HashSet<>();
            triggerSet.add(trigger.build());

            scheduler.scheduleJob(jobDetail, triggerSet, true);
        } catch (SchedulerException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }
    }

    @Override
    @ApiOperation(value = "修改任务状态")
    public void updateSysTaskTimeStatus(Long id, int status, SessionUser user) {
        LOGGER.info("定时任务 SYS_MENU  id:{} status:{} user:{}", id,status,user);
        SysTaskTime taskTime = taskTimeRDao.getById(id);
        taskTime.setTaskStatus(status);
        taskTime.setModifyUser(user.getUserName());
        taskTimeRDao.update(taskTime);

        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(taskTime.getName(), taskTime.getGroupName());
            if (!scheduler.checkExists(triggerKey)) {
            }

            if (status == CommonEnumUtil.DATA_STATUS.ENABLE.getValue()){
                scheduler.resumeTrigger(triggerKey);
            } else {
                scheduler.pauseTrigger(triggerKey);
            }
        } catch (SchedulerException e) {
            LOGGER.info("重新开始任务-->复杂调度" + e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    @ApiOperation(value = "根据根据id删除数据")
    public void deleteSysTaskTimeById(Long id) {
        try {
            SysTaskTime taskTime = taskTimeRDao.getById(id);
            ValidatorBean.checkNotNull(taskTime,"定时任务不存在");

            taskTimeRDao.deleteById(id);
            // 删除关系表数据
            refExpressionTriggerRDao.deleteByProperty("expressionId",taskTime.getTimeExpressionId());

            TriggerKey triggerKey = TriggerKey.triggerKey(taskTime.getName(),taskTime.getGroupName());
            if(scheduler.checkExists(triggerKey)){
                scheduler.pauseTrigger(triggerKey);
                scheduler.unscheduleJob(triggerKey);
            }else {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("定时任务不存在")
                        .setErrorSolution("请重新选择")
                        .build();
            }
        } catch (SchedulerException e) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.TASK_EXCEPTION.getCode())
                    .build();
        }
    }

    @Override
    @ApiOperation(value = "新增定时任务")
    public SysTaskTime insertSysTaskTime(SysTaskTime taskTime, SysTaskTimeExpression taskTimeExpression) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            // 唯一校验
            JobKey jobKey = JobKey.jobKey(taskTime.getName(),taskTime.getGroupName());
            TriggerKey triggerKey = TriggerKey.triggerKey(taskTime.getName(),taskTime.getGroupName());
            if (taskTimeRDao.isExitByProperty(new String[]{"name","groupName"},new String[]{taskTime.getName(),taskTime.getGroupName()})
                    || scheduler.checkExists(jobKey) || scheduler.checkExists(triggerKey)){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                        .setErrorDetail("定时任务已存在")
                        .setErrorSolution("请重新输入任务名称和组名称")
                        .build();
            }

            SysRefExpressionTrigger refExpressionTrigger = new SysRefExpressionTrigger();
            refExpressionTrigger.setExpressionId(taskTimeExpression.getId());
            refExpressionTrigger.setTriggerName(taskTime.getName());
            refExpressionTrigger.setTriggerGroupName(taskTime.getGroupName());

            taskTime = taskTimeRDao.insert(taskTime);
            // 添加表达式与触发器关系表
            refExpressionTriggerRDao.insert(refExpressionTrigger);

            // 添加定时任务与触发器到quartz表中
            CronScheduleBuilder schedBuilder = CronScheduleBuilder
                    .cronSchedule(taskTimeExpression.getExpressionContent())
                    .withMisfireHandlingInstructionDoNothing();
            TriggerBuilder<CronTrigger> trigger = TriggerBuilder.newTrigger()
                    .withIdentity(triggerKey)
                    .withDescription(taskTimeExpression.getExpressionDescription())
                    .withSchedule(schedBuilder)
                    .startAt(sdf.parse(taskTimeExpression.getExpressionStartDatetime()));
            if(taskTimeExpression.getExpressionEndDatetime() != null){
                trigger = trigger.endAt(sdf.parse(taskTimeExpression.getExpressionEndDatetime()));
            }

            Class<? extends Job> clazz = (Class<? extends Job>) Class.forName(taskTime.getTaskPackage() + "." + taskTime.getTaskClass());
            JobDetail jobDetail = JobBuilder.newJob(clazz).withIdentity(jobKey).withDescription(taskTime.getTaskDescription()).build();

            scheduler.scheduleJob(jobDetail, trigger.build());
        } catch (SchedulerException e) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.TASK_EXCEPTION.getCode())
                    .build();
        } catch (ParseException e) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_FORMAT.getCode())
                    .setErrorDetail("表达式起止时间错误")
                    .setErrorSolution("请更改表达式起止时间")
                    .build();
        } catch (ClassNotFoundException e) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("定时任务包名及类名不正确")
                    .setErrorSolution("请重新输入定时任务包名及类名")
                    .build();
        }
        return taskTime;
    }

    @Override
    @ApiOperation(value = "查询所有数据")
    public List<SysTaskTime> listSysTaskTime() {
        LOGGER.info("定时任务 TASK_TIME find All");
        return taskTimeRDao.findAll();
    }

    @Override
    @ApiOperation(value = "根据id 数据")
    public SysTaskTime getSysTaskTimeById(Long id) {
        LOGGER.info("定时任务 TASK_TIME find id:{}", id);
        return taskTimeRDao.getOne(id);
    }

    @Override
    @ApiOperation(value = "定时任务复制查询，分页，排序")
    public ListPager querySysTaskTimeByPager(SysTaskTime taskTime, Pager pager) {
        LOGGER.info("定时任务 TASK_TIME SysTaskTime:{}，Pager:{}",taskTime,pager);
        if(taskTime == null) {
            pager = PagerHelper.getPager(pager, taskTimeRDao.listCount());
            return new ListPager(taskTimeRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysTaskTime(taskTime);
            pager = PagerHelper.getPager(pager, taskTimeRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(taskTimeRDao.findByHqlWherePage(hqlPack + taskTime.orderBy(),pager),pager);
        }
    }

    @Override
    @ApiOperation(value = "根据id批量删除定时任务")
    public void deleteSysTaskTimeByIds(Long[] ids) {
        LOGGER.info("定时任务 TASK_TIME ids:{}", ids.toString());
        taskTimeRDao.deleteByIds(ids);
    }

    @Override
    @ApiOperation(value = "批量修改定时任务状态")
    public void updateSysTaskTimeStatusByIds(Long[] ids, Integer status) {
        LOGGER.info("定时任务 TASK_TIME ids：{}，status：{}",ids,status);
        StringBuffer where = new StringBuffer();
        HqlPack.getInPack(StringUtils.join( ids,","), "id", where);

        taskTimeRDao.updateByHqlWhere(where.toString(), "taskStatus", status);
    }

    @Override
    @ApiOperation(value = "修改最后时间")
    public void doSysTaskTimeLastDateByNameAndGroupName(String lastDate, String name, String groupName) {
        taskTimeRDao.updateByProperties(
                new String[]{"name","groupName"},new String[]{name,groupName},
                "lastRunDateTime",lastDate);
    }

    @Override
    @ApiOperation(value = "立即执行")
    public void executeSysTaskTimeById(Long id) {
        SysTaskTime taskTime = taskTimeRDao.getById(id);
        JobKey jobKey = JobKey.jobKey(taskTime.getName(),taskTime.getGroupName());
        try {
            if (scheduler.checkExists(jobKey)) {
                scheduler.triggerJob(jobKey);
            } else {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("定时任务不存在")
                        .setErrorSolution("请重新输入任务名称和组名称")
                        .build();
            }
        } catch (SchedulerException e) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.TASK_EXCEPTION.getCode())
                    .build();
        }
    }
}
